#include "TextureDemo.h"
#include "DDSTextureLoader.h"

struct Vertex
{
	XMFLOAT3 pos;
	XMFLOAT2 tex;
};

TextureDemo::TextureDemo() :m_pInputLayout(nullptr), m_pVertexBuffer(nullptr),m_pFx(nullptr),m_pTechnique(nullptr),
m_pFxWorldViewProj(nullptr),m_theta(1.5f*XM_PI), m_phi(0.4f*XM_PI), m_radius(5.0f)
{
	XMMATRIX I = XMMatrixIdentity();
	XMStoreFloat4x4(&m_world, I);
	XMStoreFloat4x4(&m_view, I);
	XMStoreFloat4x4(&m_proj, I);
}

TextureDemo::~TextureDemo()
{

}

bool TextureDemo::LoadContent()
{
	HRESULT hr;

	//create vertex buffer	
	GeometryGenerator::GetInstance()->CreateBox(2, 1.5f, 2, m_box);

	std::vector<Vertex> vertices(m_box.vertices.size());
	
	for (UINT i = 0; i < m_box.vertices.size(); ++i)
	{
		vertices[i].pos = m_box.vertices[i].pos;
		vertices[i].tex = m_box.vertices[i].tex;
	}	

	D3D11_BUFFER_DESC vertexDesc;
	ZeroMemory(&vertexDesc, sizeof(vertexDesc));
	vertexDesc.Usage = D3D11_USAGE_IMMUTABLE;
	vertexDesc.BindFlags = D3D11_BIND_VERTEX_BUFFER;
	vertexDesc.ByteWidth = sizeof(Vertex)* m_box.vertices.size();
	D3D11_SUBRESOURCE_DATA resourceData;
	ZeroMemory(&resourceData, sizeof(resourceData));
	resourceData.pSysMem = &vertices[0];
	hr = m_pd3dDevice->CreateBuffer(&vertexDesc, &resourceData, &m_pVertexBuffer);
	if (FAILED(hr))
	{
		return false;
	}
	//set vertex buffer
	UINT stride = sizeof(Vertex);
	UINT offset = 0;
	m_pImmediateContext->IASetVertexBuffers(0, 1, &m_pVertexBuffer, &stride, &offset);
	//set primitive topology
	m_pImmediateContext->IASetPrimitiveTopology(D3D11_PRIMITIVE_TOPOLOGY_TRIANGLELIST);

	//create index buffer
	std::vector<UINT> indices(m_box.indices.size());
	
	for (UINT i = 0; i < m_box.indices.size(); ++i)
	{
		indices[i] = m_box.indices[i];
	}
	
	D3D11_BUFFER_DESC indexDesc;
	ZeroMemory(&indexDesc, sizeof(indexDesc));
	indexDesc.Usage = D3D11_USAGE_IMMUTABLE;
	indexDesc.BindFlags = D3D11_BIND_INDEX_BUFFER;
	indexDesc.ByteWidth = sizeof(UINT)* m_box.indices.size();

	D3D11_SUBRESOURCE_DATA indexData;
	ZeroMemory(&indexData, sizeof(indexData));
	indexData.pSysMem = &indices[0];
	hr = m_pd3dDevice->CreateBuffer(&indexDesc, &indexData, &m_pIndexBuffer);
	if (FAILED(hr))
	{
		return false;
	}

	//set index buffer
	m_pImmediateContext->IASetIndexBuffer(m_pIndexBuffer, DXGI_FORMAT_R32_UINT, 0);

	//create shader resource view
	//ԭȵD3DX11CreateShaderResourceViewFromFileѾˣƼʹ
	//ʹDirectXTexеCreateDDSTextureFromFileͼ
	hr = CreateDDSTextureFromFile(m_pd3dDevice, L"Texture/Wood.dds", nullptr, &m_pTexSRV);
	if (FAILED(hr))
	{
		MessageBox(nullptr, L"create texture failed!", L"error",MB_OK);
		return hr;
	}

	//compile shader
	ID3DBlob* errorBlob;
	DWORD shaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;	
#if defined _DEBUG || defined DEBUG
	shaderFlags = D3DCOMPILE_DEBUG;
#endif
	hr = D3DX11CompileEffectFromFile(L"FX/texture.fx", nullptr, D3D_COMPILE_STANDARD_FILE_INCLUDE, shaderFlags,
		0, m_pd3dDevice, &m_pFx, &errorBlob);
	if (FAILED(hr))
	{
		MessageBox(nullptr, L"create shader failed!", L"error", MB_OK);
		return hr;
	}	
	m_pTechnique = m_pFx->GetTechniqueByName("TextureTech");
	m_pFxWorldViewProj = m_pFx->GetVariableByName("g_worldViewProj")->AsMatrix();
	m_pFxTex = m_pFx->GetVariableByName("g_tex")->AsShaderResource();
	
	
	//define the input layout
	D3D11_INPUT_ELEMENT_DESC layout[] =
	{
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 }
	};

	UINT numLayoutElements = ARRAYSIZE(layout);
	D3DX11_PASS_DESC passDesc;
	m_pTechnique->GetPassByIndex(0)->GetDesc(&passDesc);

	//create the input layout
	hr = m_pd3dDevice->CreateInputLayout(layout, numLayoutElements, passDesc.pIAInputSignature,
		passDesc.IAInputSignatureSize, &m_pInputLayout);
	if (FAILED(hr))
		return hr;

	//set input layout
	m_pImmediateContext->IASetInputLayout(m_pInputLayout);

	return true;
}

void TextureDemo::UnLoadContent()
{
	if (m_pVertexBuffer) m_pVertexBuffer->Release();
	if (m_pIndexBuffer)	m_pIndexBuffer->Release();
	if (m_pInputLayout)	m_pInputLayout->Release();
	//if (m_pFx) m_pFx->Release();
}

void TextureDemo::Update(float dt)
{
	float x = m_radius*sinf(m_phi)*cosf(m_theta);
	float z = m_radius*sinf(m_phi)*sinf(m_theta);
	float y = m_radius*cosf(m_phi);


	XMVECTOR pos = XMVectorSet(x, y, z, 1.0f);
	XMVECTOR target = XMVectorSet(0.0f, 0.0f, 0.0f, 1.0f);
	XMVECTOR up = XMVectorSet(0.0f, 1.0f, 0.0f, 0.0f);

	XMMATRIX V = XMMatrixLookAtLH(pos, target, up);
	XMStoreFloat4x4(&m_view, V);
	XMMATRIX T = XMMatrixPerspectiveFovLH(XM_PIDIV4, m_width / static_cast<float>(m_height),
		1.0f, 1000.0f);
	XMStoreFloat4x4(&m_proj, T);	

	//set constant buffer
	XMMATRIX world = XMLoadFloat4x4(&m_world);
	XMMATRIX view = XMLoadFloat4x4(&m_view);
	XMMATRIX proj = XMLoadFloat4x4(&m_proj);
	XMMATRIX worldViewProj = world*view*proj;

	//ȫֱ
	m_pFxTex->SetResource(m_pTexSRV);
	m_pFxWorldViewProj->SetMatrix(reinterpret_cast<float*>(&worldViewProj));
}

void TextureDemo::Render()
{
	if (m_pImmediateContext == 0)
		return;
	//clear render target view
	float clearColor[4] = { 0.75f, 0.75f, 0.75f, 1.0f };
	m_pImmediateContext->ClearRenderTargetView(m_pRenderTargetView, clearColor);
	
	//clear depth/stencil view
	m_pImmediateContext->ClearDepthStencilView(m_pDepthStencilView, D3D11_CLEAR_DEPTH | D3D11_CLEAR_STENCIL,
		1.0f, 0);

	D3DX11_TECHNIQUE_DESC techDesc;
	m_pTechnique->GetDesc(&techDesc);

	for (UINT i = 0; i < techDesc.Passes; ++i)
	{
		m_pTechnique->GetPassByIndex(i)->Apply(0, m_pImmediateContext);
		m_pImmediateContext->DrawIndexed(m_box.indices.size(), 0, 0);
	}
	m_pSwapChain->Present(0, 0);
}

void TextureDemo::OnMouseDown(WPARAM btnState, int x, int y)
{
	m_lastMousePos.x = x;
	m_lastMousePos.y = y;
	SetCapture(m_hWnd);
}

void TextureDemo::OnMouseUp(WPARAM btnState, int x, int y)
{
	ReleaseCapture();
}

//numberlowhigh֮ 
template<typename T>
static T Clamp(const T& x, const T& low, const T& high)
{
	return x < low ? low : (x > high ? high : x);
}

void TextureDemo::OnMouseMove(WPARAM btnState, int x, int y)
{
	if ((btnState & MK_LBUTTON) != 0)
	{
		//Ƕת
		float dx = XMConvertToRadians(0.25f*static_cast<float>(x - m_lastMousePos.x));
		float dy = XMConvertToRadians(0.25f*static_cast<float>(y - m_lastMousePos.y));

		m_theta -= dx;
		m_phi -= dy;

		m_phi = Clamp(m_phi, 0.1f, XM_PI - 0.1f);
	}
	else if ((btnState & MK_RBUTTON) != 0)
	{
		float dx = 0.2f*static_cast<float>(x - m_lastMousePos.x);
		float dy = 0.2f*static_cast<float>(y - m_lastMousePos.y);

		m_radius += dx - dy;

		m_radius = Clamp(m_radius, 3.0f, 300.0f);
	}

	m_lastMousePos.x = x;
	m_lastMousePos.y = y;
}


